import numpy as np
from tifffile import imsave

from tracking_challenge_demo import napari_get_reader


def test_reader_gt(tmpdir):
    """Test gt folders with no sequence are read correctly"""

    # write some fake data using your supported file format
    gt_pth = tmpdir.mkdir("01_GT").mkdir("SEG")

    gt_labels = np.random.randint(0, 20, size=(100, 100), dtype=np.uint8)
    gt_tif = gt_pth.join("man_seg000.tif")
    imsave(str(gt_tif), gt_labels)

    # try to read it back in
    reader = napari_get_reader(gt_pth)
    assert callable(reader)

    # make sure we're delivering the right format
    layer_data_list = reader(str(gt_pth))
    assert isinstance(layer_data_list, list) and len(layer_data_list) == 1
    layer_data_tuple = layer_data_list[0]
    assert (
        isinstance(layer_data_tuple, tuple) and layer_data_tuple[2] == "labels"
    )

    # make sure it's the same as it started
    layer_data = layer_data_tuple[0]
    assert layer_data.shape == (1, 100, 100)
    assert all([np.allclose(gt_labels, im_slice) for im_slice in layer_data])


def test_reader_gt_w_seq(tmpdir):
    """Test gt frames with sister sequence are read correctly"""

    seq_pth = tmpdir.mkdir("01")
    original_data = np.random.randint(
        0, 2 ** 12, size=(100, 100), dtype=np.uint16
    )
    seq_tif = seq_pth.join("t000.tif")
    seq2_tif = seq_pth.join("t001.tif")
    imsave(str(seq_tif), original_data)
    imsave(str(seq2_tif), original_data)

    gt_pth = tmpdir.mkdir("01_GT").mkdir("SEG")

    gt_labels = np.random.randint(0, 20, size=(100, 100), dtype=np.uint8)
    gt_tif = gt_pth.join("man_seg001.tif")
    imsave(str(gt_tif), gt_labels)

    # try to read it back in
    reader = napari_get_reader(gt_pth)
    assert callable(reader)

    # make sure we're delivering the right format
    layer_data_list = reader(str(gt_pth))
    assert isinstance(layer_data_list, list) and len(layer_data_list) == 1
    layer_data_tuple = layer_data_list[0]
    assert (
        isinstance(layer_data_tuple, tuple) and layer_data_tuple[2] == "labels"
    )

    # make sure it's the same as it started
    layer_data = layer_data_tuple[0]
    assert layer_data.shape == (2, 100, 100)
    np.testing.assert_allclose(gt_labels, layer_data[1])


def test_get_reader_gt_pass(tmpdir):
    """Valid dir of GT tiffs"""
    seq = tmpdir.mkdir("01_GT").mkdir("SEG")
    seq.join("man_seg000.tif").write("test")

    reader = napari_get_reader(seq)
    assert callable(reader)


def test_get_reader_not_dir_fail(tmpdir):
    """Not receiving a directory"""
    seq = tmpdir.mkdir("01").join("t000.tif")
    reader = napari_get_reader(seq)

    assert reader is None


def test_get_reader_wrong_dir_name_fail(tmpdir):
    """Directory incorrectly named"""
    seq = tmpdir.mkdir("foobar").join("t000.tif")
    reader = napari_get_reader(seq)

    assert reader is None


def test_get_reader_mix_tif_fail(tmpdir):
    """Directory ok but tifs are not named by expected sequence"""
    seq = tmpdir.mkdir("01")
    seq.join("t000.tif").write("test")
    seq.join("foobar.tif").write("test")

    reader = napari_get_reader(seq)
    assert reader is None


def test_get_reader_mix_dir_fail(tmpdir):
    """Directory is sequence but files are GT or vice versa"""
    seq = tmpdir.mkdir("01")
    seq.join("man_seg000.tif").write("test")
    reader = napari_get_reader(seq)
    assert reader is None

    seq = tmpdir.mkdir("01_GT")
    seq.join("t000.tif").write("test")
    reader = napari_get_reader(seq)
    assert reader is None


def test_get_reader_no_tif_fail(tmpdir):
    """Directory is ok but no tifs are present"""
    seq = tmpdir.mkdir("01")
    seq.join("not_a_tif.jpg").write("test")

    reader = napari_get_reader(seq)
    assert reader is None



"""
This module is an example of a barebones writer plugin for napari.

It implements the Writer specification.
see: https://napari.org/plugins/guides.html?#writers

Replace code below according to your needs.
"""
import os
from shutil import make_archive, rmtree
from typing import Any

from napari.qt import thread_worker
from tifffile import imsave


def write_single_image(path: str, data: Any, meta: dict):
    if str(path).endswith(".zip"):
        path = str(path)[:-4]
    if not data.ndim == 3:
        return None

    @thread_worker()
    def write_tiffs(data, dir_pth):
        for i, t_slice in enumerate(data):
            tiff_nme = f"seg{str(i).zfill(3)}.tif"
            tiff_pth = os.path.join(dir_pth, tiff_nme)
            imsave(tiff_pth, t_slice)

    def zip_dir():
        make_archive(path, "zip", path)
        rmtree(path)

    os.mkdir(path)
    layer_dir_pth = os.path.join(path, f"01_AUTO/SEG")
    os.makedirs(layer_dir_pth)
    if len(data.shape) == 2:
        data = [data]
    worker = write_tiffs(data, layer_dir_pth)
    worker.start()
    worker.finished.connect(zip_dir)
    return path + ".zip"



try:
    from ._version import version as __version__
except ImportError:
    __version__ = "unknown"

from ._widget import SegmentationDiffHighlight, Threshold, segment_by_threshold
from ._reader import napari_get_reader
from ._writer import write_single_image

__all__ = [
    "napari_get_reader",
    "write_single_image",
    "segment_by_threshold",
    "SegmentationDiffHighlight",
    "Threshold",
]



import os
from zipfile import ZipFile

import numpy as np

from tracking_challenge_demo import write_single_image


def test_writing_single_layer(tmpdir, qtbot):
    layer_data = np.random.randint(0, 255, (5, 100, 100))
    pth = os.path.join(str(tmpdir), "test_labels.zip")
    write_single_image(pth, layer_data, {})

    def check_zip():
        assert os.path.exists(pth)
        zip_file = ZipFile(pth)
        tiff_filenames = [
            info.filename
            for info in zip_file.infolist()
            if ".tif" in info.filename
        ]
        assert len(tiff_filenames) == 5
        assert all(["01_AUTO/SEG/seg00" in fname for fname in tiff_filenames])

    # we use waitUntil to wait for the thread to finish without needing
    # its finished signal
    qtbot.waitUntil(check_zip)


def test_writing_returns_none(tmpdir):
    layer_data = np.random.randint(0, 255, (100, 100))
    pth = os.path.join(str(tmpdir), "test_labels.zip")
    assert write_single_image(pth, layer_data, {}) is None
